{% extends 'core_main_app/_render/admin/theme/tools/box.html' %}

{% block box_title %}Template XSLT Rendering{% endblock %}

{% block box_tools %}

{% include 'core_main_app/common/buttons/go_to.html' with url=data.url_back_to label='Back to Versions' %}

{% endblock %}

{% block box_body %}
<div id="error-div" style="display: none;">
    <div class="alert alert-danger alert-dismissable">
        <a href="#" class="close" data-dismiss="alert" aria-label="close">&times;</a>
        <strong>{{ data.errors }}</strong>
    </div>
</div>

<form id="form_start" method="post">
    {% csrf_token %}
    {{ data.form_template_xsl_rendering }}

    <br/>
    <button type="submit" class="btn btn-default pull-left">
        <i class="fa fa-times"></i> Cancel
    </button>
    <button type="submit" class="btn btn-default pull-right">
      <i class="fa fa-floppy-o"></i> Save
    </button>
</form>
{% endblock %}