<a href="#" class="logo">
  <span class="logo-mini"><b>A</b>DB</span>
  <span class="logo-lg"><b>Admin</b> Dashboard</span>
</a>

<nav class="navbar navbar-static-top" role="navigation">
  <a href="#" class="sidebar-toggle" data-toggle="offcanvas" role="button">
    <span class="sr-only">Toggle navigation</span>
  </a>
  <!-- Navbar Right Menu -->
  <div class="navbar-custom-menu">
    <ul class="nav navbar-nav">
      <li>
        <a href="{% url 'core_main_app_homepage' %}"><i class="fa fa-chevron-left"></i> Go to the website</a>
      </li>
      <li class="user user-menu">
        <a href="#">
          <i class="fa fa-user"></i> {{ user }}
        </a>
      </li>
    </ul>
  </div>
</nav>