<div class="container-fluid">
    <div class="row">
        <div class="col-md-12">
            <div class="col-xs-6">
                <h2>{{ data.data.title }} <small> {{ data.data.template.display_name }}</small></h2>
            </div>
            {% include 'core_main_app/common/data/tools_data.html' %}
        </div>
    </div>
</div>
{% include 'core_main_app/common/data/detail_data.html' %}