<div class="pull-right hidden-xs">
Template by <a href="https://almsaeedstudio.com/">Almsaeed studio</a>
</div>

<strong>&copy; 2012 - {% now "Y" %} <a href="#">{{ WEBSITE_SHORT_TITLE }} </a></strong> Administrative Panel.