{% extends 'core_main_app/admin/commons/upload/form.html' %}

{% block upload_addons %}
    {% if data.dependency_resolver %}
    <hr>
    <div id="dependencies">
        {{ data.dependency_resolver }}
    </div>
    {% endif %}

    <div id="vm_id" style="display: none;">{{data.version_manager.id}}</div>
    <div id="redirect_url" style="display: none;">{{data.redirect_url}}</div>
{% endblock %}