{% extends "core_main_app/_render/admin/theme/tools/section.html" %}

{% block section_title %} Edit rights {% endblock %}

{% block section_content %}
    {% include 'core_main_app/user/workspaces/edit_rights.html' %}
{% endblock %}
